
module Schpin.Package.Xml where

import Text.XML.HXT.Core
import Schpin.Shared
import Schpin.Package


cMaintainerToXML :: ArrowXml a => Person -> a XmlTree XmlTree
cMaintainerToXML (Person name mail) = mkelem "maintainer" [sattr "email" mail] [txt name]

cAuthorToXML :: ArrowXml a => Person -> a XmlTree XmlTree
cAuthorToXML (Person name mail) = mkelem "author" [sattr "email" mail] [txt name]

cPackageToXML :: ArrowXml a => PackageInfo -> a XmlTree XmlTree
cPackageToXML (PackageInfo name ver desc maints lice authors deps) = mkelem "package" [sattr "name" name] $
  [ selem "name" [ txt name ]
  , selem "version" [ txt ver ]
  , selem "description" [ txt desc ]
  , selem "license" [ txt lice ]
  ]
   ++ (cMaintainerToXML <$> maints)
   ++ (cAuthorToXML <$> authors)
   ++ ((\x -> selem "depend" [txt x]) <$> deps)

writePackageXML :: PackageConfig -> IO()
writePackageXML pcfg = do
   let filename = pkg_dir pcfg ++ "/package.xml"
   makeParentDirectories filename
   runX $ Text.XML.HXT.Core.root [] [cPackageToXML (pkg_info pcfg)] >>> writeDocument [withIndent yes] filename
   return ()
